%% CC BY-SA (Attribution-ShareAlike) Mamdani based neuro-fuzzy system with center-of-sets defuzzification 
%by Dr. Juan R. Castro of Universidad Autnoma de Baja California 
%is distributed under a Creative Commons Attribution-ShareAlike 4.0 International License 
%(https://creativecommons.org/licenses/by-sa/4.0/). 
%This license allows others to remix, retouch, and create from their work, even for commercial purposes, 
%as long as they give credit and license their new creations under the same terms. All new works based on 
%yours will carry the same license, so any derivative works will also allow commercial use.

function fis = fnnsetx(fis,X)
% FNNSETX Set all fis weight values with a single vector.
n_in  = length(fis.input);    % n inputs
n_out = length(fis.output);   % m outputs
n_rule = length(fis.rule);    % r rules

n_p1 = 2*n_in*n_rule;         % antecedent parameters
n_p2 = n_rule*n_out;          % consequent parameters
n_pt = n_p1 + n_p2;           % total parameters
Xi1  = X(1:n_p1);
Xi2  = X(n_p1+1:n_pt);
XiS  = Xi1(1:n_p1/2);
XiC  = Xi1(n_p1/2+1:n_p1);

SIGMA  = reshape(XiS,n_rule,n_in);
CENTER = reshape(XiC,n_rule,n_in);
THETA  = reshape(Xi2,n_rule,n_out);

for i=1:n_in,
    for k=1:n_rule,
        fis.input(i).mf(k).params = [SIGMA(k,i) CENTER(k,i)];
    end
end

for j=1:n_out,
    theta = reshape(THETA(:,j),1,n_rule)';
    for k=1:n_rule,
        fis.output(j).mf(k).params = theta(k,:);
    end
end
