%% CC BY-SA (Attribution-ShareAlike) Mamdani based neuro-fuzzy system with center-of-sets defuzzification 
%by Dr. Juan R. Castro of Universidad Autnoma de Baja California 
%is distributed under a Creative Commons Attribution-ShareAlike 4.0 International License 
%(https://creativecommons.org/licenses/by-sa/4.0/). 
%This license allows others to remix, retouch, and create from their work, even for commercial purposes, 
%as long as they give credit and license their new creations under the same terms. All new works based on 
%yours will carry the same license, so any derivative works will also allow commercial use.

%Model based on the hybrid architecture of the error signal through backpropagation for feedforward networks 
%specifically in ordered derivated of Werbos to calculate the gradient, 
%created in Neuro-Fuzzy and Soft Computing - A Computational Approach to Learning and Machine Intelligence 
%by Jyh-Shing Roger Jang, Chuen-Tsai Sun and Eiji Mizutani in 1997.


function [gX,normgX,epsilon] = mamcost1fnncalcgrad(FIS,X,E,a)
% mamcost1fnncalcgrad.m
[SIGMA,CENTER,THETA] = fnngetm(FIS);

r = size(CENTER,1); 
m = size(THETA,2);  

[n,q] = size(X);

dE_dM = zeros(r,n);
dE_dS = zeros(r,n);
dE_dC = zeros(r,m);

for p=1:q
    dEp_dM = zeros(r,n);
    dEp_dS = zeros(r,n);
    dEp_dC = zeros(r,m);
   
    % Layer 3
    for i3=1:m
        epsilon{3}(i3,p) = -2*E(i3,p);
    
        for i2=1:r
            dEp_dC(i2,i3) = epsilon{3}(i3,p)*a{2}(i2,p);
        end
    end
    % Layer 2
    for i2=1:r
        epsilon{2}(i2,p) = 0;
        for i3=1:m
            epsilon{2}(i2,p) = epsilon{2}(i2,p)+epsilon{3}(i3,p)*THETA(i2,i3);
        end
    end
    % Layer 1
    for i1=1:r
        tmp1 = sum(epsilon{2}(:,p).*a{2}(:,p));
        tmp2 = sum(a{1}(:,p));
        epsilon{1}(i1,p) = (epsilon{2}(i1,p)-tmp1)/tmp2;
        
        for i0=1:n
            tmp  = epsilon{1}(i1,p)*a{1}(i1,p);
            tmpm = (X(i0,p)-CENTER(i1,i0))/SIGMA(i1,i0)^2; 
            tmps = tmpm*(X(i0,p)-CENTER(i1,i0))/SIGMA(i1,i0);
           
            dEp_dM(i1,i0) = tmp*tmpm;
            dEp_dS(i1,i0) = tmp*tmps;
        end
    end
    dE_dM = dE_dM + dEp_dM;
    dE_dS = dE_dS + dEp_dS;
    dE_dC = dE_dC + dEp_dC;
end

gX = [dE_dS(:); dE_dM(:); dE_dC(:)];
normgX = norm(gX);
