%% CC BY-SA (Attribution-ShareAlike) Mamdani based neuro-fuzzy system with center-of-sets defuzzification 
%by Dr. Juan R. Castro of Universidad Autnoma de Baja California 
%is distributed under a Creative Commons Attribution-ShareAlike 4.0 International License 
%(https://creativecommons.org/licenses/by-sa/4.0/). 
%This license allows others to remix, retouch, and create from their work, even for commercial purposes, 
%as long as they give credit and license their new creations under the same terms. All new works based on 
%yours will carry the same license, so any derivative works will also allow commercial use.

%Model based on the hybrid architecture of the error signal through backpropagation for feedforward networks, 
%created in Neuro-Fuzzy and Soft Computing - A Computational Approach to Learning and Machine Intelligence 
%by Jyh-Shing Roger Jang, Chuen-Tsai Sun and Eiji Mizutani in 1997.

function [SSE,Y,E,a] = mamcost1fnncalcperf(X,T,FIS)
% evaltskt1fnn.m
%
% Compute the output(s) of a type-1 MAMCOSFLS (type-1 antecedents 
% and type-0 consequent) when the antecedent membership functions 
% are Gaussian.
%
% center, sigma are rxn matrix denotes the mean and std of
% antecedent Gaussian MFs (r rules, with n antecedent in each rule).
% theta is rxm matrix, which denotes the coefficients of consequents.
% X is input matrix, nxq matrix, each row is on input.
%
[sigma,center,theta] = fnngetm(FIS);
r = size(center,1);
m = size(theta,2);
[n,q] = size(X);
E = zeros(m,q);
SSE = 0.0;
for p=1:q
    % Layer 1. Firing strength of a rule, a{1}
    for i1=1:r
        suma = 0.0;
        for i0=1:n
            suma = suma+((X(i0,p)-center(i1,i0))/sigma(i1,i0))^2;
        end
        a{1}(i1,p)=exp(-suma/2);
    end
    % Layer 2. Normalized firing strengths, a{2}
    for i2=1:r
        a{2}(i2,p) = a{1}(i2,p)/sum(a{1}(:,p));
    end
    % Layer 3. Computes the overall output, a{3}  
    for i3=1:m
        a{3}(i3,p) = 0;
        for i2=1:r
            a{3}(i3,p) = a{3}(i3,p)+theta(i2,i3)*a{2}(i2,p);
        end
        % Compute errors (E) and sum square error (SSE)
        E(i3,p) = T(i3,p)-a{3}(i3,p);
        SSE = SSE + E(i3,p)^2;
    end
end
Y = a{3};
